from django.db import models


class FooModel(models.Model):
    description = models.CharField(max_length=200)
    image = models.FileField(
        upload_to='uploads/foomodel/img/', null=True, blank=True)
    attachment = models.FileField(
        upload_to='uploads/attachments/', null=True, blank=True)



from django.test import TestCase
from django.core.files.storage import default_storage as media_storage
from django.core.management import call_command
from django_media_fixtures import finders
from pathlib import Path


class TestCollectMedia(TestCase):

    def test_sample_media_from_app_folder_collected(self):
        """
        Default behaviour, looking for the folder media_fixtures
        inside of all registered app's folders.
        """
        IMAGE_PATH = 'uploads/foomodel/img/board.jpg'
        if media_storage.exists(IMAGE_PATH):
            media_storage.delete(IMAGE_PATH)
        self.assertEqual(media_storage.exists(IMAGE_PATH), False)
        call_command('collectmedia', verbosity=3, interactive=False)
        self.assertEqual(media_storage.exists(IMAGE_PATH), True)

    def test_sample_media_from_custom_folder_collected(self):
        """
        Testing collect media from custom file directories, defined on
        MEDIA_FIXTURES_FILES_DIRS settings variable.
        NOTE: To use this option, you need add
        'django_media_fixtures.finders.FileSystemFinder' on
        MEDIA_FIXTURES_FILES_FINDERS settings variable.
        """
        FILE_PATH = 'uploads/attachments/example_file.txt'
        if media_storage.exists(FILE_PATH):
            media_storage.delete(FILE_PATH)
        self.assertEqual(media_storage.exists(FILE_PATH), False)
        call_command('collectmedia', verbosity=3, interactive=False)
        self.assertEqual(media_storage.exists(FILE_PATH), True)

    def test_searching_media_by_fixtures_finders(self):
        """
        Testing get the files by the media_fixtures finders directly,
        not by default_storage from Django.
        """
        IMAGE_PATH = 'uploads/foomodel/img/board.jpg'
        FILE_PATH = 'uploads/attachments/example_file.txt'
        call_command('collectmedia', clear=True,
                     verbosity=3, interactive=False)
        media_finders = finders.get_finders()

        # searching for the image and the text file, where each one need a
        # different finder (AppDirectoriesFinder or FileSystemFinder)
        has_found_img = False
        has_found_file = False
        for finder in media_finders:
            if not has_found_img:
                has_found_img = finder.find(IMAGE_PATH) or False
            if not has_found_file:
                has_found_file = finder.find(FILE_PATH) or False

        self.assertTrue(has_found_img)
        self.assertTrue(has_found_file)

    def test_list_mediafiles_by_fixtures_finders(self):
        """Simple test to assert finders can list the files."""
        call_command('collectmedia', verbosity=3, interactive=False)
        media_finders = finders.get_finders()
        files_found = []
        for finder in media_finders:
            for file in finder.list():
                files_found.append(file)
        self.assertTrue(files_found)

    def test_symbolic_link_on_collectmedia(self):
        """Simple test to assert collectmedia can make symbolic links."""
        call_command('collectmedia', link=True, clear=True,
                     verbosity=3, interactive=False)
        media_finders = finders.get_finders()
        files_found = []
        for finder in media_finders:
            for file in finder.list():
                files_found.append(file)
                self.assertTrue(Path(media_storage.path(file[0])).is_symlink())
        self.assertTrue(files_found)



import os
from collections import OrderedDict
from functools import lru_cache

from django.apps import apps
from django.conf import settings
from django.contrib.staticfiles import utils
from django.core.exceptions import ImproperlyConfigured
from django.core.files.storage import FileSystemStorage
from django.utils._os import safe_join
from django.utils.module_loading import import_string

from django.contrib.staticfiles.finders import BaseFinder


# To keep track on which directories the finder has searched the media files.
searched_locations = []


class FileSystemFinder(BaseFinder):
    """
    A media files finder that uses the ``MEDIA_FIXTURES_FILES_DIRS`` setting
    to locate files.
    """

    def __init__(self, app_names=None, *args, **kwargs):
        # List of locations with media_fixtures files
        self.locations = []
        # Maps dir paths to an appropriate storage instance
        self.storages = OrderedDict()
        if not isinstance(settings.MEDIA_FIXTURES_FILES_DIRS, (list, tuple)):
            raise ImproperlyConfigured(
                "Your MEDIA_FIXTURES_FILES_DIRS setting is not a tuple or "
                " list; perhaps you forgot a trailing comma?")
        for root in settings.MEDIA_FIXTURES_FILES_DIRS:
            if isinstance(root, (list, tuple)):
                prefix, root = root
            else:
                prefix = ''
            if settings.MEDIA_ROOT and (
                    os.path.abspath(settings.MEDIA_ROOT)
                    == os.path.abspath(root)):
                raise ImproperlyConfigured(
                    "The MEDIA_FIXTURES_FILES_DIRS setting should "
                    "not contain the MEDIA_ROOT setting")
            if (prefix, root) not in self.locations:
                self.locations.append((prefix, root))
        for prefix, root in self.locations:
            filesystem_storage = FileSystemStorage(location=root)
            filesystem_storage.prefix = prefix
            self.storages[root] = filesystem_storage
        super(FileSystemFinder, self).__init__(*args, **kwargs)

    def find(self, path, all=False):
        """
        Looks for files in the extra locations
        as defined in ``MEDIA_FIXTURES_FILES_DIRS``.
        """
        matches = []
        for prefix, root in self.locations:
            if root not in searched_locations:
                searched_locations.append(root)
            matched_path = self.find_location(root, path, prefix)
            if matched_path:
                if not all:
                    return matched_path
                matches.append(matched_path)
        return matches

    def find_location(self, root, path, prefix=None):
        """
        Finds a requested media file in a location, returning the found
        absolute path (or ``None`` if no match).
        """
        if prefix:
            prefix = '%s%s' % (prefix, os.sep)
            if not path.startswith(prefix):
                return None
            path = path[len(prefix):]
        path = safe_join(root, path)
        if os.path.exists(path):
            return path

    def list(self, ignore_patterns=None):
        """
        List all files in all locations.
        """
        for prefix, root in self.locations:
            storage = self.storages[root]
            for path in utils.get_files(storage, ignore_patterns):
                yield path, storage


class AppDirectoriesFinder(BaseFinder):
    """
    A media fixture files finder that looks in the directory of each app as
    specified in the source_dir attribute.
    """
    storage_class = FileSystemStorage
    source_dir = 'media_fixtures'
    if hasattr(settings, 'MEDIA_FIXTURE_FOLDERNAME'):
        source_dir = settings.MEDIA_FIXTURE_FOLDERNAME

    def __init__(self, app_names=None, *args, **kwargs):
        # The list of apps that are handled
        self.apps = []
        # Mapping of app names to storage instances
        self.storages = OrderedDict()
        app_configs = apps.get_app_configs()
        if app_names:
            app_names = set(app_names)
            app_configs = [ac for ac in app_configs if ac.name in app_names]
        for app_config in app_configs:
            app_storage = self.storage_class(
                os.path.join(app_config.path, self.source_dir))
            if os.path.isdir(app_storage.location):
                self.storages[app_config.name] = app_storage
                if app_config.name not in self.apps:
                    self.apps.append(app_config.name)
        super(AppDirectoriesFinder, self).__init__(*args, **kwargs)

    def list(self, ignore_patterns=None):
        """
        List all files in all app storages.
        """
        for storage in self.storages.values():
            if storage.exists(''):  # check if storage location exists
                for path in utils.get_files(storage, ignore_patterns):
                    yield path, storage

    def find(self, path, all=False):
        """
        Looks for files in the app directories.
        """
        matches = []
        for app in self.apps:
            app_location = self.storages[app].location
            if app_location not in searched_locations:
                searched_locations.append(app_location)
            match = self.find_in_app(app, path)
            if match:
                if not all:
                    return match
                matches.append(match)
        return matches

    def find_in_app(self, app, path):
        """
        Find a requested media file in an app's media fixtures locations.
        """
        storage = self.storages.get(app, None)
        if storage:
            # only try to find a file if the source dir actually exists
            if storage.exists(path):
                matched_path = storage.path(path)
                if matched_path:
                    return matched_path


def get_finders():
    """
    Set the media fixtures finders on settings.py.
    Example:
        MEDIA_FIXTURES_FILES_FINDERS = (
            'django_media_fixtures.finders.FileSystemFinder',
            'django_media_fixtures.finders.AppDirectoriesFinder',  # default
        )
    """
    if hasattr(settings, 'MEDIA_FIXTURES_FILES_FINDERS'):
        finders = settings.MEDIA_FIXTURES_FILES_FINDERS
    else:
        finders = (
            'django_media_fixtures.finders.AppDirectoriesFinder',
        )

    for finder_path in finders:
        yield get_finder(finder_path)


@lru_cache(maxsize=None)
def get_finder(import_path):
    """
    Imports the media fixtures files finder class described by import_path,
    where import_path is the full Python path to the class.
    """
    Finder = import_string(import_path)
    if not issubclass(Finder, BaseFinder):
        raise ImproperlyConfigured('Finder "%s" is not a subclass of "%s"' %
                                   (Finder, BaseFinder))
    return Finder()



import os

# BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(os.path.abspath(__file__))


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '&(6%yjbvjnn!k138ai^_fck-i@w*7on=nrin6rk+3zj83g8rec'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'django_media_fixtures',  # include bellow django.contrib.staticfiles
    'django_media_fixtures.tests.testapp.apps.TestAppConfig',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

MIDDLEWARE = MIDDLEWARE_CLASSES  # Django 2.2+


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'mem_db',
    }
}

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = '/static/'

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

# Media fixtures configurations

MEDIA_FIXTURES_FILES_FINDERS = (
    # combined with MEDIA_FIXTURES_FILES_DIRS, choose specific folders
    'django_media_fixtures.finders.FileSystemFinder',
    # default (if you do not set MEDIA_FIXTURES_FILES_FINDERS)
    'django_media_fixtures.finders.AppDirectoriesFinder',
)
MEDIA_FIXTURES_FILES_DIRS = [
    os.path.join(BASE_DIR, 'extra_media_fixtures'),
]



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

import codecs
import os
import re
import sys

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the relevant file
with codecs.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

def read(*parts):
    # intentionally *not* adding an encoding option to open, See:
    #   https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return codecs.open(os.path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")    

setup(
    name='sweelol',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=find_version('django_media_fixtures','__init__.py'),

    description='Simple project to copy media files (intended for fixtures loads), pretty much as Django staticfiles does',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/pojoba02/django-media-fixtures.git',

    # Author details
    author='Francis I AM',
    author_email='pojoba01@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 4.0',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        'Programming Language :: Python :: 3 :: Only',
    ],

    # What does your project relate to?
    keywords='django collectmedia mediafiles mediafixtures fixtures media',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['Django>=1.8'],
    python_requires='>=3.5',

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'sample=sample:main',
        ],
    },
    zip_safe=False
)
